# Electioncoefs2.R

# Election counter coefficients for the model 2's that include fiscal
# and monetary accountability.  Compares both time periods.
#
# Patrick T. Brandt
#
# 2007-01-15 : Initial version
# 2008-01-25 : Revised for updated models and code base of MSBVAR
# 2008-12-06 : Revised to label densities with the new equation names.

library(MSBVAR)
source("hidden.R")

# Function to draw the coefficients conditional on A(0)

"mcmc.structural.szbvar" <- function(varobj, A0.posterior)
  {
    m<-dim(varobj$ar.coefs)[1]  # Capture the number of variablesbcoefs <- varobj$Bhat
    p<-dim(varobj$ar.coefs)[3]    # Capture the number of lags

    ncoef <- dim(varobj$B.posterior)[1]
    n0 <- varobj$n0
    n0cum <- c(0,cumsum(n0))
    N2 <- A0.posterior$N2

  # Get the covar for the coefficients
    XXinv <- chol(solve(varobj$Hpinv.posterior[[1]]))

  # storage for the sampled coefficients.
    B.sample <- matrix(0, nrow=N2, ncol=ncoef*m)

  # Loop for the impulse responses
    for(i in 1:N2)
    {
      # Set up the A0 for this iteration
        A0 <- A0.get(A0.posterior$A0.posterior, i)
        A0inv <- solve(A0)

        bj <- a2b(A0, varobj$Ui)

        F.draw <- matrix(0, ncoef, m)

        for(j in 1:m)
        { btmp <- bj[(n0cum[j]+1):(n0cum[(j+1)])]
          F.draw[,j] <- varobj$P.posterior[[j]]%*%(btmp)
        }

        F.draw <- F.draw + XXinv%*%matrix(rnorm(m*ncoef), ncoef, m)
        B.draw <- F.draw%*%(A0inv)

        B.sample[i,] <- matrix(B.draw, 1, ncoef*m)
        if (i%%1000==0)
        { cat("Monte Carlo Iteration = ", i, "\n"); }
    }

    # output / return
    output <- list(B.sample=B.sample)
    class(output) <- c("mcmc.bsvar.posterior")
    return(output)

  }


# Load the A(0) matrices for each of the models
load("M2-97.Rdata")
rm(Model2.97.impulses)
gc()

load("M2-06.Rdata")
rm(Model2.06.impulses)
gc()


# Now draw all of the regression coefficients --- these are the
# reduced form coefficients

set.seed(4687)

Model2.97.coefs <- mcmc.structural.szbvar(M2.97,
                                          M2.97.post)

Model2.06.coefs <- mcmc.structural.szbvar(M2.06,
                                          M2.06.post)


# Set in the indices for the election coefficients
elec.indices <- c(1:12)*74

# Subset out the election coefficients for each of the output
Model2.97.elec <- Model2.97.coefs$B.sample[,elec.indices]
Model2.06.elec <- Model2.06.coefs$B.sample[,elec.indices]

# Define the variable names for the equations
varnames <- c(expression(e[t]), expression(r[t]^d),
	      expression(r[t]^i), expression(g[t]),
	      expression(p[t]^d), expression(y[t]^d),
	      expression(p[t]^i), expression(y[t]^i),
	      expression(ne[t]), expression(pe[t]),
	      expression(a[t]), expression(v[t]))

pdf(file="Electioncoefs2.pdf", width=5, height=7)

par(mfcol=c(6,2), mai=c(0.3,0.6,0.25,0.25))
for (i in 1:12)
{ density1 <- density(Model2.97.elec[,i])
  density2 <- density(Model2.06.elec[,i])

  rangex <- range(density1$x, density2$x)
  rangey <- range(density1$y, density2$y)

  plot(density1, ylim=rangey, xlim=rangex, main=varnames[i],
       lty=1, ylab="")
  lines(density2, lty=2)
  abline(v=0)

}
dev.off()

save(Model2.97.coefs,Model2.06.coefs,
     file="Electioncoefs2.RData")

q(save="no")
