# IRFsubsetrsSFB2.R
#
#
# Generates plots of subsets of the IRFs for the models
#
# Patrick T. Brandt
# pbrandt@utdallas.edu
#
# 20070105 : Initial version
# 20070107 : Added IRFs for macropolity responses to production
#            sector.
# 20081206 : Modified set of impulse responses and updated labels of
#            the plots to match text in the latest draft of the paper.
# 20081231 : Modified IRF subsets to use 12 months rather than 24
#            months for the final figures.  Matches our earlier CPS
#            paper.


# New function that uses a list of MCMC generated impulse responses
# and knows how to combine them into one set of graphs with smart
# coloring / line types.

plot.mc.irf.svar.subset <- function(mc.impulse.list, num.eqn, num.shk,
                                    probs = c(0.16, 0.84),
                                    method="Percentile",
                                    component=1,
                                    eqnnames = NULL,
                                    shknames = NULL, ...)

{
    total <- num.eqn*num.shk
    m <- num.eqn
    nsteps <- dim(mc.impulse.list[[1]])[2]
    draws <- dim(mc.impulse.list[[1]])[1]

    # Now build a set of lists of arrays to store the irf summaries
    irf.storage <- array(0, c(nsteps, length(probs) + 1, total,
                              length(mc.impulse.list)))

    irf.ci <- array(0, c(nsteps, length(probs) + 1, total))

    if(method == "Percentile") {
        for (j in 1:length(mc.impulse.list))
        {
            for (i in 1:total) {
                irf.bands <- t(apply(mc.impulse.list[[j]][, , i], 2, quantile,
                                     probs))
                irf.mean <- apply(mc.impulse.list[[j]][, , i], 2, mean)
                irf.ci[, , i] <- cbind(irf.bands, irf.mean)
            }
            irf.storage[,,,j] <- irf.ci
        }
    }

    if(method == "Sims-Zha2")
    {          eigen.sum <- matrix(0, total, nsteps)
        for (j in 1:length(mc.impulse.list))
        {
            for (i in 1:total) {
            decomp <- eigen(var(mc.impulse.list[[j]][, , i]), symmetric = T)
            W <- decomp$vectors
            lambda <- decomp$values
            gammak <- mc.impulse.list[[j]][, , i] * (W[component, ])
            gammak.quantiles <- t(apply(gammak, 2, quantile,
                probs = probs))
            irf.mean <- apply(mc.impulse.list[[j]][, , i], 2, mean)
            irf.bands <- irf.mean + gammak.quantiles
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
            eigen.sum[i, ] <- 100 * lambda/sum(lambda)

            }
            irf.storage[,,,j] <- irf.ci
        }
    }

    # Clean up
    rm(irf.ci)
    gc()

    # Now find the min-max values for the plots
    minmax <- array(0, c(num.eqn, 2, length(mc.impulse.list)))


    if(num.eqn>1)
    {
        for (j in 1:length(mc.impulse.list))
        {
            within.plots <- apply(irf.storage[,,,j], 3, range)
            tmp <- (c(1:total)%%num.eqn)
            tmp[tmp == 0] <- num.eqn
            indices <- sort(tmp, index.return = T)$ix
            dim(indices) <- c(num.shk, num.eqn)
            for (i in 1:num.eqn) {
                minmax[i,,j] <- range(within.plots[, indices[, i]])
            }
        }
    }else{ minmax <- array(range(irf.storage), c(num.eqn, 2,
    length(mc.impulse.list))) }

    # Now do the plotting
    j <- 1
    par(mfcol = c(num.eqn, num.shk), mai = c(0.15, 0.2, 0.25, 0.15), omi = c(0.25,
        0.85, 1, 0.15))
    for (i in 1:total) {
        lims <- ifelse((i-num.eqn)%%num.eqn==0, num.eqn,
                       (i-num.eqn)%%num.eqn)
        # Grabs the i'th response for the j'th model
        tmp <- matrix(irf.storage[, , i,], nrow=nsteps,
                      ncol=3*dim(irf.storage)[4])

        ts.plot(tmp,
                gpars = list(xlab = "",
                lty=rep(seq(1:length(mc.impulse.list)), 1, each=3),
                ylab = "", #col=c(rep("black",3), rep("red", 3)),
                lwd=1,
                ylim = 1.1*range(minmax[lims,,])))
        abline(h = 0)
        if(i<=num.eqn)
        { mtext(eqnnames[i], side=2, line=3) }
        if((i-1)%%num.eqn==0)
        { mtext(shknames[j], side=3, line=2)
          j <- j+1
      }

    }
    mtext("Response in", side = 2, line = 3, outer = T)
    mtext("Shock to", side = 3, line = 3, outer = T)
    cat("Eigendecomposition variation\n")
    cat("Percentage of variation explained by component", component, "\n")
    print(eigen.sum[,component])
}



plot.mc.irf.svar.subset.fixedscale <- function(mc.impulse.list, num.eqn, num.shk,
                                    probs = c(0.16, 0.84),
                                    eqnnames = NULL,
                                    shknames = NULL,
                                    method="Percentile",
                                    component=1,
                                    ...)

{
    total <- num.eqn*num.shk
    m <- num.eqn
    nsteps <- dim(mc.impulse.list[[1]])[2]
    draws <- dim(mc.impulse.list[[1]])[1]

    # Now build a set of lists of arrays to store the irf summaries
    irf.storage <- array(0, c(nsteps, length(probs) + 1, total,
                              length(mc.impulse.list)))

    irf.ci <- array(0, c(nsteps, length(probs) + 1, total))

    if (method == "Percentile") {
    for (j in 1:length(mc.impulse.list))
    { if (total>1){
        for (i in 1:total) {
            irf.bands <- t(apply(mc.impulse.list[[j]][, , i], 2, quantile,
                                 probs))
            irf.mean <- apply(mc.impulse.list[[j]][, , i], 2, mean)
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
        }
    } else { for (i in 1:total) {
        irf.bands <- t(apply(mc.impulse.list[[j]], 2, quantile,
                             probs))
        irf.mean <- apply(mc.impulse.list[[j]], 2, mean)
        irf.ci[, , i] <- cbind(irf.bands, irf.mean)
    }
         }
      irf.storage[,,,j] <- irf.ci
  }
}

    if (method == "Sims-Zha2") {
      for (j in 1:length(mc.impulse.list))
        {
          for (i in 1:total) {
             if (total>1) {
                 decomp <- eigen(var(mc.impulse.list[[j]][, , i]),
                                 symmetric = T)
                 W <- decomp$vectors
                 lambda <- decomp$values
                 gammak <- mc.impulse.list[[j]][, , i] * (W[component, ])
                 gammak.quantiles <- t(apply(gammak, 2, quantile,
                                             probs = probs))
                 irf.mean <- apply(mc.impulse.list[[j]][, , i], 2,
                                   mean)
                 # print eigen %
                 cat("Eigen %", (100*lambda/sum(lambda))[1], "\n")

             } else {
                 decomp <- eigen(var(mc.impulse.list[[j]]),
                 symmetric=T)
                          W <- decomp$vectors
                 lambda <- decomp$values
                 gammak <- mc.impulse.list[[j]] * (W[component, ])
                 gammak.quantiles <- t(apply(gammak, 2, quantile,
                                             probs = probs))
                 irf.mean <- apply(mc.impulse.list[[j]], 2, mean)

                 cat("Eigen %", (100*lambda/sum(lambda))[1], "\n")

             }

            irf.bands <- irf.mean + gammak.quantiles
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
         }

          irf.storage[,,,j] <- irf.ci
      }
    }

    # Clean up
    rm(irf.ci)
    gc()

    # Now find the min-max values for the plots
    minmax <- array(0, c(num.eqn, 2, length(mc.impulse.list)))

##     for (j in 1:length(mc.impulse.list))
##     {
##         within.plots <- apply(irf.storage[,,,j], 3, range)
##         tmp <- (c(1:total)%%num.eqn)
##         tmp[tmp == 0] <- num.eqn
##         indices <- sort(tmp, index.return = T)$ix
##         dim(indices) <- c(num.shk, num.eqn)
##         for (i in 1:num.eqn) {
##             minmax[i,,j] <- range(within.plots[, indices[, i]])
##         }
##     }

    if(num.eqn>1)
    {
        for (j in 1:length(mc.impulse.list))
        {
            within.plots <- apply(irf.storage[,,,j], 3, range)
            tmp <- (c(1:total)%%num.eqn)
            tmp[tmp == 0] <- num.eqn
            indices <- sort(tmp, index.return = T)$ix
            dim(indices) <- c(num.shk, num.eqn)
            for (i in 1:num.eqn) {
                minmax[i,,j] <- range(within.plots[, indices[, i]])
            }
        }
    }else{ minmax <- array(range(irf.storage), c(num.eqn, 2,
    length(mc.impulse.list))) }

    # Now do the plotting
    j <- 1
    par(mfcol = c(num.eqn, num.shk), mai = c(0.25, 0.25, 0.15, 0.25),
        omi = c(0.35, 0.75, 1, 0.15), cex=0.9)
    for (i in 1:total) {
        lims <- ifelse((i-num.eqn)%%num.eqn==0, num.eqn,
                       (i-num.eqn)%%num.eqn)

        tmp <- matrix(irf.storage[, , i,], nrow=nsteps,
                      ncol=3*dim(irf.storage)[4])

        ts.plot(tmp,
                gpars = list(xlab = "",
                lty=rep(seq(1:length(mc.impulse.list)), 1, each=3),
                ylab = "", #col=c(rep("black",3), rep("red", 3)),
                lwd=1,
                ylim = c(min(minmax[lims,,]), 0.1)))
        abline(h = 0)
        if(i<=num.eqn)
        { mtext(eqnnames[i], side=2, line=3) }
        if((i-1)%%num.eqn==0)
        { mtext(shknames[j], side=3, line=2)
          j <- j+1
      }

    }
    mtext("Response in", side = 2, line = 3, outer = T)
    mtext("Shock to", side = 3, line = 3, outer = T)
}


# Define the subsets of IRFs we want to extract
## - IR and G reaction to political evaluations 1984-1997 and 1997-2006
## - Political evaluations' reactions to IR and G 1984-1997 and 1997-2006
## - CPI and IIP reactions to G and IR 1984-1997 and 1997-2006
## - Political evaluations reactions to CPI and IIP

IRG <- c(98, 100, 110, 112)
MacroP2Policy <- c(23, 24, 47, 48)
Prod2Policy <- c(17, 18, 41, 42)
MacroP2Prod <- c(59, 60, 71, 72)

# Load model objects and keep what we need and
# extract subsets of IRFs
load("M2-97.RData")
rm(M2.97.A0)

IR.M2.97 <- 100*M2.97.impulses[,1:12,IRG]
MacroP2Policy.M2.97 <- 100*M2.97.impulses[,1:12,MacroP2Policy]
Prod2Policy.M2.97 <- 100*M2.97.impulses[,1:12,Prod2Policy]
MacroP2Prod.M2.97 <- 100*M2.97.impulses[,1:12,MacroP2Prod]

rm(M2.97.impulses)

load("M2-06.RData")
rm(M2.06.A0)

IR.M2.06 <- 100*M2.06.impulses[,1:12, IRG]
MacroP2Policy.M2.06 <- 100*M2.06.impulses[,1:12,MacroP2Policy]
Prod2Policy.M2.06 <- 100*M2.06.impulses[,1:12,Prod2Policy]
MacroP2Prod.M2.06 <- 100*M2.06.impulses[,1:12,MacroP2Prod]

rm(M2.06.impulses)

## # Get the signs of the diagonals for the A0's so we can decide how to
## # flip signs for interpretation.
## A0diagonals <- cbind(diag(model2.97$A0.mode), diag(model2.06$A0.mode))
## rownames(A0diagonals) <- colnames(Y)
## colnames(A0diagonals) <- c("1984-1997","1997-2005")

# 68% error band versions

pdf(file="Figure1-68.pdf", width=5, height=5)
plot.mc.irf.svar.subset(list(IR.M2.97, IR.M2.06), num.eqn=2, num.shk=2,
                        probs = c(0.16, 0.84), method="Sims-Zha2",
                        eqnnames = c(expression(r[t]^d), expression(g[t])),
                        shknames = c(expression(ne[t]), expression(pe[t])))
dev.off()

pdf(file="Figure2-68.pdf", width=5, height=5)
plot.mc.irf.svar.subset(list(MacroP2Policy.M2.97,
                             MacroP2Policy.M2.06), num.eqn=2,
                        num.shk=2,
                        probs = c(0.16, 0.84), method="Sims-Zha2",
                        shknames = c(expression(r[t]^d), expression(g[t])),
                        eqnnames = c(expression(a[t]), expression(v[t])))
dev.off()

pdf(file="Figure3-68.pdf", width=5, height=5)
plot.mc.irf.svar.subset(list(Prod2Policy.M2.97,
                             Prod2Policy.M2.06),
                        num.eqn=2, num.shk=2,
                        probs = c(0.16, 0.84), method="Sims-Zha2",
                        shknames = c(expression(r[t]^d), expression(g[t])),
                        eqnnames = c(expression(p[t]^d), expression(y[t]^d)))
dev.off()

pdf(file="Figure4-68.pdf", width=5, height=5)
plot.mc.irf.svar.subset(list(MacroP2Prod.M2.97,
                             MacroP2Prod.M2.06),
                        num.eqn=2, num.shk=2,
                        probs = c(0.16, 0.84), method="Sims-Zha2",
                        eqnnames = c(expression(a[t]), expression(v[t])),
                        shknames = c(expression(p[t]^d), expression(y[t]^d)))
dev.off()


# 90% error band versions

pdf(file="Figure1-90.pdf", width=5, height=5)
plot.mc.irf.svar.subset(list(IR.M2.97, IR.M2.06), num.eqn=2, num.shk=2,
                        probs = c(0.05, 0.95), method="Sims-Zha2",
                        eqnnames = c(expression(r[t]^d), expression(g[t])),
                        shknames = c(expression(ne[t]), expression(pe[t])))
dev.off()

pdf(file="Figure2-90.pdf", width=5, height=5)
plot.mc.irf.svar.subset(list(MacroP2Policy.M2.97,
                             MacroP2Policy.M2.06), num.eqn=2,
                        num.shk=2,
                        probs = c(0.05, 0.95), method="Sims-Zha2",
                        shknames = c(expression(r[t]^d), expression(g[t])),
                        eqnnames = c(expression(a[t]), expression(v[t])))
dev.off()

pdf(file="Figure3-90.pdf", width=5, height=5)
plot.mc.irf.svar.subset(list(Prod2Policy.M2.97,
				     Prod2Policy.M2.06),
                        num.eqn=2, num.shk=2,
                        probs = c(0.05, 0.95), method="Sims-Zha2",
                        shknames = c(expression(r[t]^d), expression(g[t])),
                        eqnnames = c(expression(p[t]^d), expression(y[t]^d)))
dev.off()

pdf(file="Figure4-90.pdf", width=5, height=5)
plot.mc.irf.svar.subset(list(MacroP2Prod.M2.97,
                             MacroP2Prod.M2.06),
                        num.eqn=2, num.shk=2,
                        probs = c(0.05, 0.95), method="Sims-Zha2",
                        eqnnames = c(expression(a[t]), expression(v[t])),
                        shknames = c(expression(p[t]^d), expression(y[t]^d)))
dev.off()


q(save="no")
