# Model2dfev.R
# Computes DFEVs for the Model 2 -- Full Representation case
#
# Patrick T. Brandt
#
# 20081219 : Initial version

library(MSBVAR)

# Load the posterior samples and drop things we do not need to free
# memory

load("M2-97.Rdata")
rm(M2.97.impulses, M2.97.A0)

load("M2-06.Rdata")
rm(M2.06.impulses, M2.06.A0)

# Generate DFEV's based on the mode from the optimization
M2.97.dfev <- dfev(M2.97, k=24)
M2.06.dfev <- dfev(M2.06, k=24)

print(M2.97.dfev, latex=T, file="M297dfev.tex")
print(M2.06.dfev, latex=T, file="M206dfev.tex")

# Now do this in a look for the draws of A(0) so we can assess the
# uncertainty of the DFEV

# The following code samples from the posterior of the reduced form
# and uses this with the A(0) draws to compute the DFEVs for the
# model.

source("~/MSBVARlocal/devel-unstable/MSBVAR/R/hidden.R")

dfevBSVAR <- function(varobj, A0.posterior, k)
{
    vartmp <- varobj            # Placeholder, since we will repop for
                                # the calls below

    m<-dim(varobj$ar.coefs)[1]  # Capture the number of variablesbcoefs <- varobj$Bhat
    p<-dim(varobj$ar.coefs)[3]  # Capture the number of lags

    ncoef <- dim(varobj$B.posterior)[1]
    n0 <- varobj$n0
    n0cum <- c(0,cumsum(n0))
    N2 <- A0.posterior$N2

  # Get the covar for the coefficients
    XXinv <- chol(solve(varobj$Hpinv.posterior[[1]]))

  # storage for the DFEV
    dfev.tmp <- dfev.tmp2 <- array(0, c(k, m, m))

  # Loop for the sampling
    for(i in 1:N2)
    {
      # Set up the A0 for this iteration
        A0 <- A0.get(A0.posterior$A0.posterior, i)
        A0inv <- solve(A0)

        bj <- a2b(A0, varobj$Ui)

        F.draw <- matrix(0, ncoef, m)

        for(j in 1:m)
        { btmp <- bj[(n0cum[j]+1):(n0cum[(j+1)])]
          F.draw[,j] <- varobj$P.posterior[[j]]%*%(btmp)
        }

        F.draw <- F.draw + XXinv%*%matrix(rnorm(m*ncoef), ncoef, m)
        B.draw <- F.draw%*%(A0inv)

        # Now put these into a matrix of AR coefs so we can pass them
        # into the DFEV function

        ar.coefs <- t(B.draw[1:(m*p),])
        dim(ar.coefs) <- c(m,m,p)
        ar.coefs <- aperm(ar.coefs, c(2,1,3))

        # See, this is were we build the varobj for an iteration.
        vartmp$ar.coefs <- ar.coefs

        # Now compute the DFEV and store a running sum and the square.

        dfev.i <- dfev(vartmp, A0inv, k)
        dfev.tmp <- dfev.tmp + dfev.i$errors
        dfev.tmp2 <- dfev.tmp2 + dfev.i$errors^2

        if (i%%1000==0)
        { cat("Monte Carlo Iteration = ", i, "\n"); }

    }

    # output / return
    dfev.tmp <- dfev.tmp/N2
    dfev.tmp2 <- dfev.tmp2/N2

    # Now compute the std. errors
    dfev.se <- sqrt(dfev.tmp2 - dfev.tmp^2)

    # Clean up the NAN in the se's -- since some are logically zero.
    dfev.se <- ifelse(is.na(dfev.se)==T, 0, dfev.se)

    output <- list(errors=dfev.tmp, std.err=dfev.se, names=colnames(varobj$y))
    class(output) <- c("dfev")
    return(output)

}


# Now do the DFEV computations
system.time(dfev1 <- dfevBSVAR(M2.97, M2.97.post, k=12))
system.time(dfev2 <- dfevBSVAR(M2.06, M2.06.post, k=12))

# Now let's plot this stuff.  Notice this is going to work like an IRF
# plot.

m <- ncol(Y)

pdf(file="Model2DFEV.pdf", width=11, height=8)

par(mfrow=c(m,m), mai = c(0.15, 0.15, 0.15, 0.15),
    omi = c(0.15, 0.75, 1, 0.15), cex.axis=0.85)

for(i in 1:m)
{
    for(j in 1:m)
    {
     ts.plot(ts(cbind(dfev1$errors[,j,i],
                   dfev2$errors[,j,i])),
          gpars=list(ylab="", xlab="", lty=1:2, lwd=2, axes=FALSE, ylim=c(0,100)))

     axis(side=1, at=seq(0, 12, by=2))
     # Labels
     if(j==1){ mtext(dfev1$names[i], side=2, line=3); axis(side=2, at=c(0,50,100))}
     if(i==1) mtext(dfev1$names[j], side=3, line=2)
    }
}

mtext("Forecast Error Variance % for Equation",
      side=2, line=4, outer=T)
mtext("Shock to",
      side=3, line=3, outer=T)

dev.off()

# Save things
save.image("Model2DFEVdata.RData")

# Now print out the DFEVs

"print.dfev.BSVAR" <- function(x, latex=F, file=NULL, ...)
{   dfev.obj <- x
    errors <- dfev.obj$errors
    names <- dfev.obj$names
    std.err <- dfev.obj$std.err
    k <- dim(errors)[1]
    m <- dim(errors)[2]

    if(latex==T){
        for (i in 1:m){
            tmp <- matrix(errors[,,i], nrow=k, ncol=m)
#            tmp <- cbind(std.err[,i],tmp)
#            colnames(tmp) <- c("Std. Error", names)
            colnames(tmp) <- c(names)
            if(i==1){
                if(is.null(file)){
                    print(xtable(tmp, digits=rep(1,ncol(tmp)+1),
                                 caption=paste("Decomposition of Forecast Errors for a Shock to", names[i])),
                          append=F, table.placement="p")
                } else {

                      # Ensure we clobber any old file
                    print(xtable(tmp,
                                 digits=rep(1,ncol(tmp)+1),
                                 caption=paste("Decomposition of Forecast Errors for a Shock to", names[i])),
                          file=file, append=F, table.placement="p")
                }
            } else {
                if(is.null(file)){
                    print(xtable(tmp,
                                 digits=rep(1,ncol(tmp)+1),
                                 caption=paste("Decomposition of Forecast Errors for a Shock to", names[i])),
                          append=T, table.placement="p")
                } else {
                    print(xtable(tmp,
                                 digits=rep(1,ncol(tmp)+1),
                                 caption=paste("Decomposition of Forecast Errors for a Shock to", names[i])),
                          file=file, append=T, table.placement="p")
                }
            }
        }
    } else {
        for (i in 1:m){
            cat(paste("Decomposition of Forecast Errors for a Shock to", names[i], "\n"))
            cat("-------------------------------------------------------------\n")
            tmp <- matrix(errors[,,i], nrow=k, ncol=m)
#            tmp <- cbind(std.err[,i],tmp)
#            colnames(tmp) <- c("Std. Error", names)
            colnames(tmp) <- c(names)
            print(tmp)
            cat("-------------------------------------------------------------\n")
        }
    }
}

print.dfev.BSVAR(dfev1, latex=T, file="M297dfevBSVAR.tex")
print.dfev.BSVAR(dfev2, latex=T, file="M206dfevBSVAR.tex")


q(save="no")
