# IRFsubsetrsSFB2.R
#
#
# Generates plots of subsets of the IRFs for the models
#
# Patrick T. Brandt
# pbrandt@utdallas.edu
#
# 20070105 : Initial version
# 20070107 : Added IRFs for macropolity responses to production
#            sector.
# 20070216 : Revised indexing of the responses to deal with the
#            smaller scale of these models


# New function that uses a list of MCMC generated impulse responses
# and knows how to combine them into one set of graphs with smart
# coloring / line types.

plot.mc.irf.svar.subset <- function(mc.impulse.list, num.eqn, num.shk,
                                    probs = c(0.16, 0.84),
                                    method="Percentile",
                                    component=1,
                                    eqnnames = NULL,
                                    shknames = NULL, ...)

{
    total <- num.eqn*num.shk
    m <- num.eqn
    nsteps <- dim(mc.impulse.list[[1]])[2]
    draws <- dim(mc.impulse.list[[1]])[1]

    # Now build a set of lists of arrays to store the irf summaries
    irf.storage <- array(0, c(nsteps, length(probs) + 1, total,
                              length(mc.impulse.list)))

    irf.ci <- array(0, c(nsteps, length(probs) + 1, total))

    if(method == "Percentile") {
        for (j in 1:length(mc.impulse.list))
        {
            for (i in 1:total) {
                irf.bands <- t(apply(mc.impulse.list[[j]][, , i], 2, quantile,
                                     probs))
                irf.mean <- apply(mc.impulse.list[[j]][, , i], 2, median)
                irf.ci[, , i] <- cbind(irf.bands, irf.mean)
            }
            irf.storage[,,,j] <- irf.ci
        }
    }

    if(method == "Sims-Zha2")
    {          eigen.sum <- matrix(0, total, nsteps)
        for (j in 1:length(mc.impulse.list))
        {
            for (i in 1:total) {
            decomp <- eigen(var(mc.impulse.list[[j]][, , i]), symmetric = T)
            W <- decomp$vectors
            lambda <- decomp$values
            gammak <- mc.impulse.list[[j]][, , i] * (W[component, ])
            gammak.quantiles <- t(apply(gammak, 2, quantile,
                probs = probs))
            irf.mean <- apply(mc.impulse.list[[j]][, , i], 2, median)
            irf.bands <- irf.mean + gammak.quantiles
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
            eigen.sum[i, ] <- 100 * lambda/sum(lambda)

            }
            irf.storage[,,,j] <- irf.ci
        }
    }

    # Clean up
    rm(irf.ci)
    gc()

    # Now find the min-max values for the plots
    minmax <- array(0, c(num.eqn, 2, length(mc.impulse.list)))


    if(num.eqn>1)
    {
        for (j in 1:length(mc.impulse.list))
        {
            within.plots <- apply(irf.storage[,,,j], 3, range)
            tmp <- (c(1:total)%%num.eqn)
            tmp[tmp == 0] <- num.eqn
            indices <- sort(tmp, index.return = T)$ix
            dim(indices) <- c(num.shk, num.eqn)
            for (i in 1:num.eqn) {
                minmax[i,,j] <- range(within.plots[, indices[, i]])
            }
        }
    }else{ minmax <- array(range(irf.storage), c(num.eqn, 2,
    length(mc.impulse.list))) }

    # Now do the plotting
    j <- 1
    par(mfcol = c(num.eqn, num.shk), mai = c(0.15, 0.2, 0.25, 0.15), omi = c(0.25,
        0.85, 1, 0.15))
    for (i in 1:total) {
        lims <- ifelse((i-num.eqn)%%num.eqn==0, num.eqn,
                       (i-num.eqn)%%num.eqn)

        tmp <- matrix(irf.storage[, , i,], nrow=nsteps,
                      ncol=3*dim(irf.storage)[4])

        ts.plot(tmp,
                gpars = list(xlab = "",
                lty=rep(seq(1:length(mc.impulse.list)), 1, each=3),
                ylab = "", #col=c(rep("black",3), rep("red", 3)),
                lwd=1,
                ylim = 1.1*range(minmax[lims,,])))
        abline(h = 0)
        if(i<=num.eqn)
        { mtext(eqnnames[i], side=2, line=3) }
        if((i-1)%%num.eqn==0)
        { mtext(shknames[j], side=3, line=2)
          j <- j+1
      }

    }
    mtext("Response in", side = 2, line = 3, outer = T)
    mtext("Shock to", side = 3, line = 3, outer = T)
    cat("Eigendecomposition variation\n")
    cat("Percentage of variation explained by component", component, "\n")
    print(eigen.sum[,component])
}


# Define the subsets of IRFs we want to extract
## - IR reaction to political evaluations
## - G reaction to political evaluations
## - Political evaluations' reactions to IR
## - Political evaluations' reactions to G
## - CPI and IIP reactions to G and IR
## - Political evaluations reactions to IIP and CPI

IR <- c(82, 92)
G <- c(84, 94)
MacroP2Policy <- c(19:20, 39:40)
Prod2Policy <- c(10, 11, 20, 21)
MacroP2Prod <- c(49:50, 59:60)

# Load model objects and keep what we need and
# extract subsets of IRFs

load("Model2.06.RData")
rm(Model2.06.A0.posterior.obj)

IR.Model2.06 <- 100*Model2.06.impulses[,, IR]
G.Model2.06 <- 100*Model2.06.impulses[,, G]
MacroP2Policy.Model2.06 <- 100*Model2.06.impulses[,,MacroP2Policy]
Prod2Policy.Model2.06 <- 100*Model2.06.impulses[,,Prod2Policy]
MacroP2Prod.Model2.06 <- 100*Model2.06.impulses[,,MacroP2Prod]

rm(Model2.06.impulses)

## # Get the signs of the diagonals for the A0's so we can decide how to
## # flip signs for interpretation.
## A0diagonals <- cbind(diag(model2.97$A0.mode), diag(model2.06$A0.mode))
## rownames(A0diagonals) <- colnames(Y)
## colnames(A0diagonals) <- c("1984-1997","1997-2005")

pdf(file="IR2Macrop.pdf")
plot.mc.irf.svar.subset(list(IR.Model2.06), num.eqn=1, num.shk=2,
                        probs = c(0.16, 0.84), method="Sims-Zha2",
                        eqnnames = c("IR"),
                        shknames = c("PM", "VI"))
dev.off()

pdf(file="G2Macrop.pdf")
plot.mc.irf.svar.subset(list(G.Model2.06), num.eqn=1, num.shk=2,
                        probs = c(0.16, 0.84), method="Sims-Zha2",
                        eqnnames = c("G"),
                        shknames = c("PM", "VI"))
dev.off()

pdf(file="MacroP2Policy.pdf")
plot.mc.irf.svar.subset(list(MacroP2Policy.Model2.06), num.eqn=2,
                        num.shk=2,
                        probs = c(0.16, 0.84), method="Sims-Zha2",
                        shknames = c("IR", "G"),
                        eqnnames = c("PM", "VI"))
dev.off()

pdf(file="Prod2Policy.pdf")
plot.mc.irf.svar.subset(list(Prod2Policy.Model2.06),
                        num.eqn=2, num.shk=2,
                        probs = c(0.16, 0.84), method="Sims-Zha2",
                        shknames = c("IR", "G"),
                        eqnnames = c("CPI", "IIP"))
dev.off()

pdf(file="MacroP2Prod.pdf")
plot.mc.irf.svar.subset(list(MacroP2Prod.Model2.06),
                        num.eqn=2, num.shk=2,
                        probs = c(0.16, 0.84), method="Sims-Zha2",
                        eqnnames = c("PM", "VI"),
                        shknames = c("CPI", "IIP"))
dev.off()

q(save="no")
